# -*- coding: utf-8 -*-
## PloneChat
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
    Manages messages sent by users
"""
__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from DateTime import DateTime

# CMF Imports
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

class ChatMessages:
    "Chat Messages Tool class"
    security = ClassSecurityInfo()

    security.declarePrivate('formatResponse')
    def formatResponse(self, moderated, items):
        """
        items --> list of mappings {'date' : 'date in chatDateFormat',
                                    'id'   : 'message id',
                                    'member' : 'username',
                                    'message' : 'message text'}
        """
        tool = self.portal_plone_chat
        userid = tool.getCurrentMemberId()        
        data = ''
        if self.displayWithTabs:
            data = '<chat moderation="%s">' % moderated
        data += self.getChatUsers() + '\t'
        for item in items:            
            item['message'].replace("\x00", "")
            if self.displayWithTabs:
                data += '<msg author="%(member)s" id="%(id)s" date="%(date)s">%(message)s</msg>' % item
            else:
                data += '<div id="%s"><label>%s</label>' % (item['id'], item['date'].split(' ')[1],)
                if not moderated:
                    data += '<input type="checkbox" name="%s" />' % item['id']
                if item['member'] == userid:
                    data += '<strong class="mine">%(member)s : </strong>%(message)s' % item
                else:
                    data += '<strong>%(member)s :</strong>%(message)s' % item
                data += '</div>'
        if self.displayWithTabs:
            data += '</chat>'
            self.REQUEST.RESPONSE.setHeader("Content-Type", "text/xml; charset=%s" % self.getCharset())
        self.REQUEST.RESPONSE.setHeader('Content-Length', len(data))
        return data
    
    security.declarePrivate('getMessages')
    def getMessages(self, moderated = True, lastID = 0):
        """ get messages """
        self.updateChatUsers()
        tool = self.portal_plone_chat
        new_list = tool.getMessages(moderated, self)
        try:
            index = map(lambda x:x['id'], new_list).index(lastID)
            new_list = new_list[index + 1:]
        except:
            pass
        lastID = len(new_list) and new_list[-1]['id'] or 0        
        return self.formatResponse(moderated, new_list)    

    security.declareProtected(CMFCorePermissions.View, 'insertNewMessage')
    def insertNewMessage(self, messageToSend = ''):
        """
        insert a new message
        """
        self.updateChatUsers();
        tool = self.portal_plone_chat
        messageToSend = messageToSend \
            .replace('<', '&lt;').replace('>', '&gt;').replace('\n', '') \
            .replace('<br />', '').replace('<br>', '').replace('<br/>', '')
        tool.insertNewMessage(messageToSend, self)
        return True    
    
    security.declareProtected(CMFCorePermissions.View, 'sendMessage')
    def sendMessage(self, fromUser, toUser, message):
        """ add a new message in storage """
        tool = getToolByName(self, TOOL_ID)
        return tool.chat_storage.insertNewMessage(message, fromUser, self)  