# -*- coding: utf-8 -*-
## PloneChat
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
    PloneChat
"""
__docformat__ = 'restructuredtext'

# Python imports
import random, string
import os.path, time
from cStringIO import StringIO

# Zope imports
from AccessControl import ClassSecurityInfo
from DateTime import DateTime

# CMF imports
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions

# Archetypes imports
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

# Product imports
from Products.PloneChat.config import *
from Products.PloneChat.content.ChatUserTracker import ChatUserTracker
from Products.PloneChat.content.ChatMessages import ChatMessages
from Products.PloneChat.content.schemata import PloneChatSchema


class PloneChat(BaseContent, ChatUserTracker, ChatMessages):
    """PloneChat class"""

    meta_type                = 'PloneChat'
    archetype_name            = 'Chat'
    content_icon                = 'PloneChat.gif'
    default_view                = 'PloneChat_view'
    schema                    = PloneChatSchema
    isPrincipiaFolderish        = True

    actions = (
        {'id'           : 'view',
         'name'         : 'View',
         'action'       : 'string:${object_url}/PloneChat_view',
         'permissions'  : (CMFCorePermissions.View, ),
         'category'     : 'object'
         },
        {'id'           : 'moderation',
         'name'         : 'Moderation',
         'action'       : 'string:${object_url}/PloneChat_moderationForm',
         'permissions'  : (PloneChat_moderatePermission, ),
         'category'     : 'object'
         },
        {'id'           : 'members',
         'name'         : 'Chat members',
         'action'       : 'string:${object_url}/folder_localrole_form',
         'permissions'  : (CMFCorePermissions.ManageProperties, ),
         'category'     : 'object'
         },
    )
    
    security = ClassSecurityInfo()

    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """
        # We set the object creator as a Reviewer - as it won't be possible to do so
        # later because of permission restrictions
        obj = self
        tool = getToolByName(self, TOOL_ID)
        member_id = tool.getCurrentMemberId()
        roles = list(obj.get_local_roles_for_userid( userid=member_id ))
        if "Reviewer" not in roles:
            roles.append( "Reviewer" )
            obj.manage_setLocalRoles( member_id, roles )
        # We reindex the object
        self.reindexObject()

    security.declarePublic("show_id")
    def show_id(self,):
        """
        Determine whether to show an id in an edit form

        show_id.py used to be a Python Script, but was removed from Plone 2.5,
        so we have to add the method here for compatability
        """
        if self.plone_utils.isIDAutoGenerated(self.REQUEST.get('id', None) or self.getId()):
            return not (self.portal_factory.isTemporary(self) or self.CreationDate() == self.ModificationDate())
        return

    # ###############
    # Interface
    # ###############
    security.declareProtected(CMFCorePermissions.View, 'getModeratedMessages')
    def getModeratedMessages(self, lastID = 0):
        """ get displayed messages """
        return self.getMessages(True, lastID)
    
    security.declareProtected(PloneChat_moderatePermission, 'getUnmoderatedMessages')
    def getUnmoderatedMessages(self, lastID = 0):
        """ get all messages (for moderation) """
        return self.getMessages(False, lastID)
        
    security.declareProtected(PloneChat_moderatePermission, 'moderateMessages')
    def moderateMessages(self, messages, mode):
        """ moderate messages """
        tool = getToolByName(self, TOOL_ID)
        tool.moderateMessages(messages, mode, self)
        return True      

    security.declareProtected(CMFCorePermissions.ManagePortal, 'clearLogs')
    def clearLogs(self):
        """
        remove all messages from the logs
        """
        tool = getToolByName(self, TOOL_ID)
        tool.clearLogs(self.UID())
        return True

    security.declareProtected(CMFCorePermissions.View, 'getLogs')
    def getLogs(self):
        """
        return all the moderated messages
        """
        tool = getToolByName(self, TOOL_ID)
        return tool.getLogs(self)

    # ##############
    # utils
    # ##############
    security.declarePrivate('getDefaultPloneSiteCharset')
    def getDefaultPloneSiteCharset(self):
        """ """
        return self.portal_properties.site_properties.getProperty('default_charset')

    security.declareProtected(CMFCorePermissions.View, "canPost")
    def canPost(self,):
        """
        canPost(self,) => Return 1 if the user can post something on the chat
        """
        mtool = getToolByName(self, 'portal_membership')
        if mtool.checkPermission(PloneChat_unrestrictedChatPermission, self) or \
               mtool.checkPermission(PloneChat_restrictedChatPermission, self):
            return 1
        return 0
    
    security.declareProtected(CMFCorePermissions.View, "canChat")
    def canChat(self,):
        """
        canChat(self,) => return 0 if the post will be moderated or post is not allowed, 1 if post won't be moderated
        """
        if not self.getModeration():
            return 1
        mtool = getToolByName(self, 'portal_membership')
        if mtool.checkPermission(PloneChat_unrestrictedChatPermission, self):
            return 1
        return 0


registerType(PloneChat, PROJECTNAME)
